using System;

namespace Test
{
	using OOCodeDOM;

	class Class1
	{
		[STAThread]
		static void Main(string[] args)
		{
			CompileUnit cu = new CompileUnit();
			Namespace ns = new Namespace("Test");

			cu += ns;

			Type t = new Type("C1");

			ns += t;

			Method m = new Method("run",
				new If((new Constant(5) ^ new Constant(6)) == new Constant(4),
				new Return(new Constant(5)[new Constant(6), new Constant(8)]),
				new Return(new Constant(4))));

			t += m;

			Console.WriteLine(cu.Source(Language.CSharp));
		}
	}
}
